#include <cpctelera.h>
#include "define.h"
#include "type.h"
#include "tile.h"
#include "data.h"
#include "player.h"
#include "tileList.h"
#include "sprite.h"
#include "group.h"
#include "permanent.h"

void screen64();
void calcOffsets();
void playerInput();
void checkCollision();
void clearBuffer();
void printSprite();
void placeObjects();
void printTiles();
void printPlayer();
void rotateBuffers();
void flipScreen();
void collisionReact();
void menuInit();
void gameInit();
void instructionsInit();
void creditsInit();
void printEnergy();
void printScore();
void printHighScore();
void printFoodScore();
void printWord_score();
void printWord_high();
void updateScore();
void printMenu();
void printMenuTitle();
void printInstructions();
void printCredits();
void printGameOverLose();
void printGameOverWin();
void waitForKey();
void gameOverWinInit();
void gameOverLoseInit();
void goToMenu();
void highScoreInit();
void scoreInit();
void flashBorder(u8,u8);
void startupActive();
void printString(u8,u8,u8*,u16,u8);
void printChar(u8,u8,u8);
void activateGroup();
// void check_respawn();
// void check_toggle();
void deactivateGroup();
void processFrame();
// void getSfxChannel();
void silence();

u8* pBufferFront;
u8* pBufferBack;
u8* pBuffer;
u8* pScreen;
u8* pTileList;
u8* vmem;

u8 page;
u8 buffer;

u8 pLeft;
u8 pRight;
u8 pTop;
u8 pBottom;
u8 pDir;
u8 pRot;
u8 cDir;

u8 pEnergyRed;
u8 pEnergyYellow;
u8 pEnergyGreen;
u8 pEnergyReps;

u8 gameState;
u8 pState;
u8 pPhase;
u8 pJumpTurn;

u8 viewLeft;
u8 viewTop;

u8 xOffset;
u8 yOffset;

u8 spriteLeft;
u8 spriteRight;
u8 spriteTop;
u8 spriteBottom;
u8 spriteWidth;
u8 spriteHeight;

u8 spriteLeftVisible;
u8 spriteRightVisible;
u8 spriteTopVisible;
u8 spriteBottomVisible;

u8 viewLeftOffset;
u8 viewRight;
u8 viewTopOffset;
u8 viewBottom;

u8 pColLeft;
u8 pColTop;

u8 sLeft;
u8 sRight;
u8 sTop;
u8 sBottom;

// u16 tileCount;

u8 score[5];
u8 highScore[5];
u16 energy;
u16 wordPos;

u8 pZone1;
u8 pZone2;
u8 pZone3;

u8 pX1;
u8 pX2;
u8 pX3;

// u8 fxChannel;
u8 fxNoteIndex;
// u8 fxGroupIndex;
u8 fxCountdown;
u8 fxLastNote;

const typeZoneInstance* znInst1;
const typeZoneInstance* znInst2;
const typeZoneInstance* znInst3;
const typeZoneInstance* znInst4;
const typeZoneInstance* znInst5;
const typeGroup* gp;
const typeGroup* gp_tmp;
typeSprite* znSprite;
typeSprite* znSprite1;
const typeCollisionRect* cList;
const typeZone* zn1;
const typeZone* zn2;
const typeZone* zn3;
u8 znCount1;
u8 znCount2;
// u8 znCount3;
u8 znCount4;
u8 znCount5;
u8 znCount6;

u8 titlePhase;
i8 titlePhaseDir;

u8 framesToDisplayFoodScore;
// u8 toggleCycles;
// u8 respawnedSomething;

u8 sRectCount;
u8 pTmp;

void main(void) {
	cpct_disableFirmware();
	cpct_setStackLocation(SCREEN_BUFFER);
 	cpct_clearScreen_f64(0);
	cpct_setVideoMode(0);
	screen64();
	highScoreInit();
	scoreInit();
	menuInit();

 	while(1){
 		if (gameState==GAME_STATE_MENU){
			printMenuTitle();
			waitForKey();

			if (cpct_isKeyPressed(Key_I))
				instructionsInit();
			else if (cpct_isKeyPressed(Key_C))
				creditsInit();
			else if (cpct_isAnyKeyPressed()){
				waitForKey();
				gameInit();
			}
 		}
 		else if (gameState==GAME_STATE_GAME){
			if (energy!=0)
				energy--;

			// respawnedSomething=0;

			// if (page==0)
			// 	check_respawn();
			// else
				// check_toggle();
			
	 		playerInput();
		    cpct_akp_musicPlay();

		    if (fxCountdown!=0)
		    	fxCountdown--;

		    if (fxCountdown==0)
		    	fxNoteIndex=0;

	 		if (gameState!=GAME_STATE_PAUSE){
	 			if (gameState!=GAME_STATE_GAME_OVER_LOSE){
					processFrame();

					// if (score[4]==5){ // 50000
					if ((score[2]==9) && (score[1]==5)){ // 950
						silence();
						cpct_akp_SFXPlay(5,15,48,0,0,AY_CHANNEL_A);
						flashBorder(0,16);
						gameOverWinInit();
					}
					else if (energy==0){
						silence();
						cpct_akp_SFXPlay(4,15,48,0,0,AY_CHANNEL_A);
						cpct_akp_SFXPlay(3,15,48,0,0,AY_CHANNEL_B);
						flashBorder(2,16);
						gameOverLoseInit();
					}
				}
			}
		}
		else if (gameState==GAME_STATE_INSTRUCTIONS)
			goToMenu();
		else if (gameState==GAME_STATE_CREDITS)
			goToMenu();
		else if (gameState==GAME_STATE_GAME_OVER_LOSE)
			goToMenu();
		else if (gameState==GAME_STATE_GAME_OVER_WIN)
			goToMenu();
		else if (gameState==GAME_STATE_PAUSE){
			__asm
			ld hl,#_lz_paused_lz
			call printWord_full
			__endasm;

			waitForKey();

			if (cpct_isKeyPressed(Key_P)){
				gameState=GAME_STATE_GAME;
				waitForKey();

				__asm
				ld hl,#_lz_goatfish_lz
				call printWord_full
				__endasm;
			}
			else if (cpct_isAnyKeyPressed()){
				gameState=GAME_STATE_GAME;

				__asm
				ld hl,#_lz_goatfish_lz
				call printWord_full
				__endasm;
			}
		}
 	}
}

void silence(){
	cpct_akp_stop();
	cpct_akp_SFXStopAll();
}

// void getSfxChannel(){
//     if (fxChannel==AY_CHANNEL_A){
//         fxChannel=AY_CHANNEL_B;
//     }
//     else if (fxChannel==AY_CHANNEL_B){
//         fxChannel=AY_CHANNEL_A;
//     }
// }

void processFrame(){
	clearBuffer();
	placeObjects();
    cpct_akp_musicPlay();
	printTiles();
    cpct_akp_musicPlay();
	printPlayer();
	printEnergy();
	printScore();
	printFoodScore();
	rotateBuffers();
	flipScreen();

    if (pState!=STATE_SWIM){
    	if (fxLastNote!=(7+31-pTop)){
	   		// getSfxChannel();
			cpct_akp_SFXPlay(1,15,fxNoteList[7+31-pTop],0,0,AY_CHANNEL_B);
			fxLastNote=7+31-pTop;
		}
	}
}

void flashBorder(u8 bdr,u8 duration){
	u8 i;
	u8 j;
	u8 b=bdr;

	for (i=0;i<duration;i++){
		for (j=0;j<8;j++){
			cpct_waitVSYNC();
			cpct_akp_musicPlay();
		}

		switch (b){
			case 0:
				cpct_setBorder(HW_BLUE);
				b=1;
				break;
			case 1:
				cpct_setBorder(HW_BRIGHT_BLUE);
				b=0;
				break;
			case 2:
				cpct_setBorder(HW_RED);
				b=3;
				break;
			case 3:
				cpct_setBorder(HW_BRIGHT_RED);
				b=2;
				break;
		}
	}

	cpct_setBorder(HW_BLUE);
}

void highScoreInit(){
	u8 i;

	for (i=0;i<5;i++)
		highScore[i]=0;

	// highScore[3]=1; // 1000 (score and highScore are stored backwards in arrays)
	highScore[2]=5; // 500 (score and highScore are stored backwards in arrays)
}

void scoreInit(){
	u8 i;

	for (i=0;i<5;i++)
		score[i]=0;
}

void goToMenu(){
	waitForKey();

	if (cpct_isAnyKeyPressed()){
		waitForKey();
		menuInit();
	}
}

void calcOffsets(){
	// if (pLeft>=192)
	// 	xOffset=128;
	// else if (pLeft<=63)
	// 	xOffset=128;
	// else
	// 	xOffset=0;

	if ((pLeft>=192) || (pLeft<=63))
		xOffset=128;
	// else if (pLeft<=63)
	// 	xOffset=128;
	else
		xOffset=0;

	// if (pTop>=192)
	// 	yOffset=128;
	// else if (pTop<=63)
	// 	yOffset=128;
	// else
	// 	yOffset=0;

	if ((pTop>=192) || (pTop<=63))
		yOffset=128;
	// else if (pTop<=63)
	// 	yOffset=128;
	else
		yOffset=0;

	viewLeft=pLeft-PLAYER_CENTRE_X;
	viewTop=pTop-PLAYER_CENTRE_Y;
}

void playerInput(){
	cpct_scanKeyboard_f();

	if (cpct_isKeyPressed(Key_P)){
		gameState=GAME_STATE_PAUSE;
		return;
	}
	else if (cpct_isKeyPressed(Key_Esc)){
		waitForKey();
		gameOverLoseInit();
		return;
	}

	if (pState==STATE_SWIM){
		switch (pDir){
			case 1:case 4:case 7:case 10:case 13:case 16:case 19:case 22:
				if ((cpct_isKeyPressed(Key_CursorLeft)) || (cpct_isKeyPressed(Joy0_Left))){
					pDir--;
					pRot=0-1;
				}

				if ((cpct_isKeyPressed(Key_CursorRight)) || (cpct_isKeyPressed(Joy0_Right))){
					pDir++;
					pRot=1;
				}

				break;
			
			default:
				pDir+=pRot;
		}

		if (pDir<1)
			pDir=24;
		else if (pDir>24)
			pDir=1;

		if ((cpct_isKeyPressed(Key_Space)) || (cpct_isKeyPressed(Joy0_Fire1))){
			switch (pDir){
				case 1:case 2:case 24:
					pTop--;
					cDir=COLLISION_UP;
					checkCollision();

					if (pTop<32){
						pState=STATE_JUMP_UP;
						pPhase=1;
					}

					break;

				case 3:case 4:case 5:
					pLeft++;
					pTop--;
					cDir=COLLISION_UP_RIGHT;
					checkCollision();

					if (pTop<32){
						pState=STATE_JUMP_UP_RIGHT;
						pPhase=1;
					}

					break;

				case 6:case 7:case 8:
					pLeft++;
					cDir=COLLISION_RIGHT;
					checkCollision();
					break;

				case 9:case 10:case 11:
					pLeft++;
					pTop++;
					cDir=COLLISION_DOWN_RIGHT;
					checkCollision();
					break;

				case 12:case 13:case 14:
					pTop++;
					cDir=COLLISION_DOWN;
					checkCollision();
					break;

				case 15:case 16:case 17:
					pLeft--;
					pTop++;
					cDir=COLLISION_DOWN_LEFT;
					checkCollision();
					break;

				case 18:case 19:case 20:
					pLeft--;
					cDir=COLLISION_LEFT;
					checkCollision();
					break;

				case 21:case 22:case 23:
					pLeft--;
					pTop--;
					cDir=COLLISION_UP_LEFT;
					checkCollision();

					if (pTop<32){
						pState=STATE_JUMP_UP_LEFT;
						pPhase=1;
					}

					break;
			}
		}
		// else{
		// 	switch (pDir){
		// 		case 1:case 2:case 24:
		// 			cDir=COLLISION_UP;
		// 			break;
		// 		case 3:case 4:case 5:
		// 			cDir=COLLISION_UP_RIGHT;
		// 			break;
		// 		case 6:case 7:case 8:
		// 			cDir=COLLISION_RIGHT;
		// 			break;
		// 		case 9:case 10:case 11:
		// 			cDir=COLLISION_DOWN_RIGHT;
		// 			break;
		// 		case 12:case 13:case 14:
		// 			cDir=COLLISION_DOWN;
		// 			break;
		// 		case 15:case 16:case 17:
		// 			cDir=COLLISION_DOWN_LEFT;
		// 			break;
		// 		case 18:case 19:case 20:
		// 			cDir=COLLISION_LEFT;
		// 			break;
		// 		case 21:case 22:case 23:
		// 			cDir=COLLISION_UP_LEFT;
		// 			break;
		// 	}

		// 	checkCollision();
		// }
	}

	else if (pState==STATE_JUMP_UP){
		switch (tblJumpUp[pPhase]){
			case 0:
				pPhase++;
				break;

			case 1:
				pPhase++;
				pTop--;
				break;

			case 2:
				pPhase++;
				pTop++;
				break;

			case 3:
				pState=STATE_SWIM;
				pTop++;

				if (pJumpTurn==0)
					pJumpTurn=1;
				else
					pJumpTurn=0;

				break;
		}

		if (pPhase>=19){
			if (pDir!=13){
				if (pJumpTurn==0)
					pDir--;
				else
					pDir++;
			}
		}

		if (pDir<1)
			pDir=24;
		else if (pDir>24)
			pDir=1;

		// cDir=COLLISION_JUMP_UP;
		checkCollision();
	}

	else if (pState==STATE_JUMP_UP_LEFT){
		switch (tblJumpUpLeftRight[pPhase]){
			case 0:
				pPhase++;
				break;

			case 1:
				pPhase++;
				pLeft--;
				pTop--;
				break;

			case 2:
				pPhase++;
				pLeft--;
				break;

			case 3:
				pPhase++;
				pLeft--;
				pTop++;
				break;

			case 4:
				pState=STATE_SWIM;
				pLeft--;
				pTop++;
				break;
		}

		if (pPhase>=17)
			if (pDir!=16)
				pDir--;

		// cDir=COLLISION_JUMP_UP_LEFT;
		checkCollision();
	}

	else if (pState==STATE_JUMP_UP_RIGHT){
		switch (tblJumpUpLeftRight[pPhase]){
			case 0:
				pPhase++;
				break;

			case 1:
				pPhase++;
				pLeft++;
				pTop--;
				break;

			case 2:
				pPhase++;
				pLeft++;
				break;

			case 3:
				pPhase++;
				pLeft++;
				pTop++;
				break;

			case 4:
				pState=STATE_SWIM;
				pLeft++;
				pTop++;
				break;
		}

		if (pPhase>=17)
			if (pDir!=10)
				pDir++;

		// cDir=COLLISION_JUMP_UP_RIGHT;
		checkCollision();
	}

	// else if (pState==STATE_FALL_DOWN){
	// 	if (pTop<32)
	// 		pTop++;

	// 	if (pTop==32)
	// 		pState=STATE_SWIM;
	// }

	// else if (pState==STATE_FALL_DOWN_LEFT){
	// 	if (pTop<32){
	// 		pLeft--;
	// 		pTop++;
	// 	}

	// 	if (pTop==32)
	// 		pState=STATE_SWIM;

	// 	cDir=COLLISION_FALL_DOWN_LEFT;
	// 	checkCollision();
	// }

	// else if (pState==STATE_FALL_DOWN_RIGHT){
	// 	if (pTop<32){
	// 		pLeft++;
	// 		pTop++;
	// 	}

	// 	if (pTop==32)
	// 		pState=STATE_SWIM;

	// 	cDir=COLLISION_FALL_DOWN_RIGHT;
	// 	checkCollision();
	// }

	calcOffsets();
}

void playerDeath(){
	// u8 z=(pTop/32)*8+pLeft/32;
	// pLeft=respawnPlayer[z].x;
	// pTop=respawnPlayer[z].y;
	// pDir=7;
	// pRot=0;
	// pState=STATE_SWIM;
	// pPhase=0;
	// pJumpTurn=0;

	if (energy>16)
		energy-=16;

	silence();
	cpct_akp_SFXPlay(4,15,48,0,0,AY_CHANNEL_A);
	cpct_akp_SFXPlay(3,15,48,0,0,AY_CHANNEL_B);
	flashBorder(2,8);
	// calcOffsets();
}

u8 hasCollided()__naked{
	// // sLeft=cList->x+znInst5->x+xOffset;

	// if (pRight>=sLeft){
	// 	// sRight=sLeft+cList->width-1;

	// 	if (pColLeft<=sRight){
	// 		// sTop=cList->y+znInst5->y+yOffset;

	// 		if (pBottom>=sTop){
	// 			// sBottom=sTop+cList->height-1;

	// 			if (pColTop<=sBottom)
	// 				return 1;
	// 		}
	// 	}
	// }

	// return 0;

	__asm

	// ld	hl, (_cList)
	// ld	a, (hl)
	ld	de, (_cList)
	ld	a, (de)
	ld	hl, (_znInst5)
	inc	hl
	inc	hl
	inc	hl
	add	a, (hl)
	ld	hl, #_xOffset
	add	a, (hl)
	ld (_sLeft),a
	ld b,a

	ld	a,(_pRight)
	// ld	hl, #_sLeft
	// sub	a, (hl)
	sub a,b
	jr	C,HC

	// ld	hl, (_cList)
	// inc	hl
	// inc	hl
	// ld	a, (hl)
	inc de
	inc de
	ld	a, (de)
	dec a
	// ld	hl,#_sLeft
	// add	a,(hl)
	add a,b
	ld	(_sRight), a

	ld	hl, #_pColLeft
	sub	a,(hl)
	jr	C,HC

	// ld	hl, (_cList)
	// inc hl
	// ld	a, (hl)
	dec de
	ld a,(de)
	ld	hl, (_znInst5)
	ld bc,#4
	add hl,bc
	add	a, (hl)
	ld	hl, #_yOffset
	add	a, (hl)
	ld (_sTop),a
	ld c,a

	ld	a,(_pBottom)
	// ld	hl, #_sTop
	// sub	a, (hl)
	sub a,c
	jr	C,HC

	// ld	hl, (_cList)
	// inc	hl
	// inc	hl
	// inc	hl
	// ld	a, (hl)
	inc de
	inc de
	ld	a, (de)
	dec a
	// ld	hl,#_sTop
	// add	a,(hl)
	add a,c
	ld	(_sBottom), a

	ld	hl, #_pColTop
	sub	a, (hl)
	jr	C,HC

	ld	l, #0x01
	ret

HC:
	ld	l, #0x00
	ret

	__endasm;
}

void a(){
	// znSprite=znInst5->sprite;
	// sRectCount=znSprite->collisionRectCount;
	// cList=znSprite->collisionList;

	__asm
	ld	hl, (_znInst5)
	ld	de, #0x0006
	add	hl, de
	ld	a, (hl)
	ld	(_znSprite), a
	ld c,a
	inc	hl
	ld	a, (hl)
	ld	(_znSprite+1), a
	ld b,a

	ld	l, c
	ld	h, b
	inc	hl
	inc	hl
	inc	hl
	ld	a, (hl)
	ld	(_sRectCount),a

	ld	l, c
	ld	h, b
	ld	de, #0x000b
	add	hl, de
	ld	a, (hl)

	ld	(_cList), a
	inc	hl
	ld	a, (hl)
	ld	(_cList+1), a
	__endasm;
}

void ccp(){
	// zn3=zone_permanent+pZone3+pX3/32;
	// znCount2=zn3->spriteCount;

	__asm

	ld	a,(_pX3)
	rlca
	rlca
	rlca
	and	a, #0x07

	ld	hl, #_pZone3
	add	a, (hl)

	ld	bc, #_zone_permanent
	ld	d, #0x00
	ld e,a
	ld	l, e
	ld	h, d
	add	hl, hl
	add	hl, de
	add	hl, bc
	ld	(_zn3), hl

	ld	a, (hl)
	ld	(_znCount2),a

	__endasm;	
}

void b(){
	// znSprite=znInst5->sprite;
	// cList=znSprite->collisionList;

	__asm
	ld	hl, (_znInst5)
	ld	de, #0x0006
	add	hl, de
	ld	a, (hl)
	ld	(_znSprite), a
	inc	hl
	ld	a, (hl)
	ld	(_znSprite+1), a

	ld	hl, (_znSprite)
	ld	de, #0x000b
	add	hl, de
	ld	a, (hl)
	ld	(_cList), a
	inc	hl
	ld	a, (hl)
	ld	(_cList+1), a
	__endasm;
}

// void ccr(){
// 	// zn3=zone_respawn+pZone3+pX3/32;
// 	// znCount3=zn3->spriteCount;

// 	__asm

// 	ld	a,(_pX3)
// 	rlca
// 	rlca
// 	rlca
// 	and	a, #0x07

// 	ld	hl, #_pZone3
// 	add	a, (hl)

// 	ld	bc, #_zone_respawn
// 	ld	d, #0x00
// 	ld e,a
// 	ld	l, e
// 	ld	h, d
// 	add	hl, hl
// 	add	hl, de
// 	add	hl, bc
// 	ld	(_zn3), hl

// 	ld	a, (hl)
// 	ld	(_znCount3),a

// 	__endasm;	
// }

// void c(){
// 	// u8 j;

// 	// for (j=0;j<GROUP_COUNT;j++){
// 	// 	gp=tblGroup[j];

// 	// 	if (gp->id==znInst5->group->id){
// 	// 		deactivateGroup();
// 	// 		break;
// 	// 	}
// 	// }
// }

// void d(){
// 	u8 j;

// 	for (j=0;j<GROUP_COUNT;j++){
// 		gp=tblGroup[j];

// 		if (gp->id==znInst5->group->nextGroupID){
// 			activateGroup();
// 			break;
// 		}
// 	}
// }

// void e(){
// 	// u8 j;

// 	// for (j=0;j<GROUP_COUNT;j++){
// 	// 	gp=tblGroup[j];

// 	// 	if (gp->id==znInst5->group->prevDoorID){
// 	// 		activateGroup();
// 	// 		break;
// 	// 	}
// 	// }
// }

// void g(){
// 	u8 j;

// 	for (j=0;j<GROUP_COUNT;j++){
// 		gp=tblGroup[j];

// 		if (gp->id==znInst5->group->nextDoorID){
// 			deactivateGroup();
// 			break;
// 		}
// 	}
// }

void ccg(){
	// zn3=zone_group+pZone3+pX3/32;
	// znCount4=zn3->spriteCount;

	__asm

	ld	a,(_pX3)
	rlca
	rlca
	rlca
	and	a, #0x07

	ld	hl, #_pZone3
	add	a, (hl)

	ld	bc, #_zone_group
	ld	d, #0x00
	ld e,a
	ld	l, e
	ld	h, d
	add	hl, hl
	add	hl, de
	add	hl, bc
	ld	(_zn3), hl

	ld	a, (hl)
	ld	(_znCount4),a

	__endasm;	
}

// void cct(){
// 	// zn3=zone_toggle+pZone3+pX3/32;
// 	// znCount5=zn3->spriteCount;

// 	__asm

// 	ld	a,(_pX3)
// 	rlca
// 	rlca
// 	rlca
// 	and	a, #0x07

// 	ld	hl, #_pZone3
// 	add	a, (hl)

// 	ld	bc, #_zone_toggle
// 	ld	d, #0x00
// 	ld e,a
// 	ld	l, e
// 	ld	h, d
// 	add	hl, hl
// 	add	hl, de
// 	add	hl, bc
// 	ld	(_zn3), hl

// 	ld	a, (hl)
// 	ld	(_znCount5),a

// 	__endasm;	
// }

u8 pTmpOffset(){
	return pTmp*32+16;
}

void checkCollision(){
	u8 x;
	u8 y;
	u8 i;
	u8 j;
	u8 pY=pTop-32;
	
	pTmp=pTop/32;

	// if (pTop>=(pTmp*32+16))
	if (pTop>=pTmpOffset())
		pY=pTop;

	pColLeft=pLeft+xOffset;
	pRight=pColLeft+PLAYER_WIDTH-1;
	pColTop=pTop+yOffset;
	pBottom=pColTop+PLAYER_HEIGHT-1;

	// for (y=0;y<3;y++){
	for (y=0;y<2;y++){
		pZone3=(pY/32)*8;
		
		// permanent

		// pX3=pLeft-64;
		pX3=pLeft-32;
		pTmp=pLeft/32;

		// if (pLeft>=(pTmp*32+16))
		if (pLeft>=pTmpOffset())
			pX3=pLeft;

		// for (x=0;x<5;x++){
		// for (x=0;x<3;x++){
		for (x=0;x<2;x++){
			ccp();

			if (znCount2!=0){
				znInst5=zn3->zoneInstance;

				for (i=0;i<znCount2;i++){
					a();

					for (j=0;j<sRectCount;j++){
						if (hasCollided()==1){
							collisionReact();
	
							if (znSprite->deadly==1)
								playerDeath();
							// else
								// collisionReact();
							return;
						}

						cList++;
					}

					znInst5++;
				}
			}

			pX3+=32;
		}

		// // respawn

		// pX3=pLeft-32;
		
		// for (x=0;x<3;x++){
		// 	ccr();

		// 	if (znCount3!=0){
		// 		znInst5=zn3->zoneInstance;

		// 		for (i=0;i<znCount3;i++){
		// 			if (znInst5->active==1){
		// 				b();

		// 				if (hasCollided()==1){
		// 					updateScore();

		// 					// if (respawnedSomething==1)
		// 					// 	processFrame();

		// 					__asm
		// 					ld hl,(_znInst5)
		// 					ld (hl),#0 // znInst7->active=0
		// 					ld de,(_znSprite)
		// 					ld a,(de)
		// 					inc hl
		// 					ld (hl),a // znInst7->inactiveCycles=znSprite->inactiveCycles
		// 					__endasm;

		// 					energy+=znSprite->energy;

		// 					if (energy>PLAYER_ENERGY_MAX)
		// 						energy=PLAYER_ENERGY_MAX;

		// 					return;
		// 				}
		// 			}

		// 			znInst5++;
		// 		}
		// 	}

		// 	pX3+=32;
		// }

		// group

		// pX3=pLeft-64;
		pX3=pLeft-32;
		pTmp=pLeft/32;
		
		// if (pLeft>=(pTmp*32+16))
		if (pLeft>=pTmpOffset())
			pX3=pLeft;

		// for (x=0;x<5;x++){
		// for (x=0;x<3;x++){
		for (x=0;x<2;x++){
			ccg();

			if (znCount4!=0){
				znInst5=zn3->zoneInstance;

				for (i=0;i<znCount4;i++){
					if (znInst5->active==1){
						b();

						if (hasCollided()==1){
							if (znSprite->door==1)
								collisionReact();
							// else if (znSprite->doorSwitch==1){
							// 	c();
							// 	d();
							// 	// e();
							// 	// getSfxChannel();
							// 	cpct_akp_SFXPlay(4,15,48,0,0,AY_CHANNEL_C);
							// }
							else{
								updateScore();

								__asm
								ld hl,(_znInst5)
								ld (hl),#0 // znInst8->active=0
								// ld de,(_znSprite)
								// ld a,(de)
								// inc hl
								// ld (hl),a // znInst8->inactiveCycles=znSprite->inactiveCycles
								__endasm;

								if (znSprite->energy!=255)
									energy+=8*znSprite->energy;
								else
									energy+=znSprite->energy;

								if (energy>PLAYER_ENERGY_MAX)
									energy=PLAYER_ENERGY_MAX;

								znInst5->group->activeCount--;

								if (znInst5->group->activeCount==0){
									// d();
									// g();
									// e();
									// getSfxChannel();

									for (j=0;j<GROUP_COUNT;j++){
										gp=tblGroup[j];

										if (gp->id==znInst5->group->nextGroupID){
											activateGroup();
											break;
										}
									}

									for (j=0;j<GROUP_COUNT;j++){
										gp=tblGroup[j];

										if (gp->id==znInst5->group->nextDoorID){
											deactivateGroup();
											break;
										}
									}

									// cpct_akp_SFXPlay(2,15,fxNoteList[fxGroupIndex],0,0,AY_CHANNEL_A);
								}
							}

							return;
						}
					}

					znInst5++;
				}
			}

			pX3+=32;
		}

		// // toggle

		// pX3=pLeft-64;
		
		// for (x=0;x<5;x++){
		// 	cct();

		// 	if (znCount5!=0){
		// 		znInst5=zn3->zoneInstance;

		// 		for (i=0;i<znCount5;i++){
		// 			if (znInst5->active==1){
		// 				b();

		// 				if (hasCollided()==1){
		// 					processFrame();
		// 					playerDeath();
		// 					return;
		// 				}
		// 			}

		// 			znInst5++;
		// 		}
		// 	}

		// 	pX3+=32;
		// }

		pY+=32;
	}
}

void updateScore(){
	u8 i;

	score[0]+=znSprite->score;

	if (score[0]>=50){
		score[1]+=5;
		score[0]-=50;
	}
	else if (score[0]>=40){
		score[1]+=4;
		score[0]-=40;
	}
	else if (score[0]>=30){
		score[1]+=3;
		score[0]-=30;
	}
	else if (score[0]>=20){
		score[1]+=2;
		score[0]-=20;
	}

	for (i=0;i<4;i++){
		if (score[i]>=10){
			score[i+1]++;
			score[i]-=10;
		}
	}

	if (score[4]==5){ // 50000
		for (i=0;i<4;i++)
			score[i]=0;
	}

	// cpct_waitVSYNC();
	// getSfxChannel();
	cpct_akp_SFXPlay(1,15,fxNoteList[fxNoteIndex],0,0,AY_CHANNEL_A);

	// fxGroupIndex=fxNoteIndex;
	fxNoteIndex++;

	if (fxNoteIndex>14)
		fxNoteIndex=14;

	fxCountdown=FRAMES_TO_RESET_NOTES;

	switch (znSprite->score){
		case 1:
			__asm
			ld hl,#_lz_apple_lz
			call printWord_full
			__endasm;
			break;
		case 2:
			__asm
			ld hl,#_lz_banana_lz
			call printWord_full
			__endasm;
			break;
		case 3:
			__asm
			ld hl,#_lz_egg_lz
			call printWord_full
			__endasm;
			break;
		case 4:
			__asm
			ld hl,#_lz_cupcake_lz
			call printWord_full
			__endasm;
			break;
		case 5:
			__asm
			ld hl,#_lz_milkshake_lz
			call printWord_full
			__endasm;
			break;
		case 6:
			__asm
			ld hl,#_lz_chicken_lz
			call printWord_full
			__endasm;
			break;
		case 7:
			__asm
			ld hl,#_lz_hamburger_lz
			call printWord_full
			__endasm;
			break;
		case 8:
			__asm
			ld hl,#_lz_pizza_lz
			call printWord_full
			__endasm;
			break;
		case 9:
			__asm
			ld hl,#_lz_boot_lz
			call printWord_full
			__endasm;
			break;
		case 10:
			__asm
			ld hl,#_lz_football_lz
			call printWord_full
			__endasm;
			break;
		case 11:
			__asm
			ld hl,#_lz_boxingGloves_lz
			call printWord_full
			__endasm;
			break;
		case 12:
			__asm
			ld hl,#_lz_teddyBear_lz
			call printWord_full
			__endasm;
			break;
		case 13:
			__asm
			ld hl,#_lz_phone_lz
			call printWord_full
			__endasm;
			break;
		case 14:
			__asm
			ld hl,#_lz_rose_lz
			call printWord_full
			__endasm;
			break;
		case 15:
			__asm
			ld hl,#_lz_aceOfSpades_lz
			call printWord_full
			__endasm;
			break;
		case 16:
			__asm
			ld hl,#_lz_loveHeart_lz
			call printWord_full
			__endasm;
			break;
		case 50:
			__asm
			ld hl,#_lz_potion_lz
			call printWord_full
			__endasm;
			break;
	}

	framesToDisplayFoodScore=FRAMES_TO_DISPLAY_FOOD_SCORE;
}

void clearBuffer(){
	cpct_memset_f64(pBufferFront,0x0000,BUFFER_WIDTH*BUFFER_HEIGHT);

	if (viewTop>0)
		if (viewTop<=32)
			cpct_memset(pBufferFront+(32-viewTop)*BUFFER_WIDTH,254,BUFFER_WIDTH); // water level is tile 254
}

void printSprite(){
	// for (y=(spriteBottom-spriteTop)+1;y>0;y--){
	// cpct_memcpy((u8*)pBuffer,(u8*)pTileList,(spriteRight-spriteLeft)+1);
	// pBuffer+=BUFFER_WIDTH;
	// pTileList+=spriteWidth;
	// }

	__asm
	di

	ld a,(_spriteLeft)
	ld b,a
	ld a,(_spriteRight)
	inc a
	sub b
	ld (PRINT_SPRITE_BC+1),a

	ld a,(_spriteTop)
	ld b,a
	ld a,(_spriteBottom)
	sub b
	inc a

	ld bc,#BUFFER_WIDTH

	ld de,(_spriteWidth)
	ld d,#0

	ld hl,(_pTileList)
	ld iy,(_pBuffer)

	LOOP_PRINT_SPRITE:

	push hl

	exx

	PRINT_SPRITE_BC:
	ld bc,#0x0000

	pop hl

	.db 0xFD
	ld d,h ; LD D,IYH
	.db 0xFD
	ld e,l ; LD E,IYL

	ldir

	exx

	add hl,de
	add iy,bc

	dec a
	jr nz,LOOP_PRINT_SPRITE

	ei
	__endasm;
}

void processSprite(){
	// a();

	// // znSprite=znInst_tmp->sprite;
	// // spriteWidth=znSprite->width;
	// // spriteLeft=znInst_tmp->x+xOffset;
	// // spriteRight=spriteLeft+spriteWidth-1;
	// // spriteLeftVisible=0;
	// // spriteRightVisible=0;

	// // if (spriteLeft>=viewLeftOffset)
	// // 	if (spriteLeft<=viewRight)
	// // 		spriteLeftVisible=1;

	// // if (spriteRight>=viewLeftOffset)
	// // 	if (spriteRight<=viewRight)
	// // 		spriteRightVisible=1;

	// if ((spriteLeftVisible==1) || (spriteRightVisible==1)){
	// 	b();

	// 	// spriteHeight=znSprite->height;
	// 	// spriteTop=znInst_tmp->y+yOffset;
	// 	// spriteBottom=spriteTop+spriteHeight-1;
	// 	// spriteTopVisible=0;
	// 	// spriteBottomVisible=0;

	// 	// if (spriteTop>=viewTopOffset)
	// 	// 	if (spriteTop<=viewBottom)
	// 	// 		spriteTopVisible=1;

	// 	// if (spriteBottom>=viewTopOffset)
	// 	// 	if (spriteBottom<=viewBottom)
	// 	// 		spriteBottomVisible=1;

	// 	if ((spriteTopVisible==1) || (spriteBottomVisible==1)){
	// 		c();

	// 		// pBuffer=pBufferFront;
	// 		// pTileList=znSprite->tileList;

	// 		if (spriteLeftVisible==0){
	// 			d();
	// 			// pTileList+=(viewLeftOffset-spriteLeft);
	// 			// spriteLeft=viewLeftOffset;
	// 		}
	// 		else{
	// 			e();
	// 			// pBuffer+=(spriteLeft-viewLeftOffset);

	// 			// if (spriteRightVisible==0)
	// 			// 	spriteRight=viewRight;
	// 		}

	// 		if (spriteTopVisible==0){
	// 			// pTileList+=(viewTopOffset-spriteTop)*spriteWidth;
	// 			// spriteTop=viewTopOffset;
	// 			f();
	// 		}
	// 		else{
	// 			// pBuffer+=(spriteTop-viewTopOffset)*BUFFER_WIDTH;

	// 			// if (spriteBottomVisible==0)
	// 			// 	spriteBottom=viewBottom;

	// 			g();
	// 		}

	// 		printSprite();
	// 	}
	// }

	__asm

	// znSprite=znInst_tmp->sprite;

	ld	hl, (_znInst1)
	ld	bc, #0x0006
	add	hl, bc
	ld	a, (hl)
	ld (_znSprite1),a
	inc	hl
	ld	a, (hl)
	ld (_znSprite1+1),a

	// spriteWidth=znSprite->width;

	ld	hl, (_znSprite1)
	inc	hl
	ld	a, (hl)
	ld	(_spriteWidth),a

	// spriteLeft=znInst_tmp->x+xOffset;

	ld	hl, (_znInst1)
	inc	hl
	inc	hl
	inc	hl
	ld	a, (hl)
	ld	hl, #_xOffset
	add	a, (hl)
	ld	(_spriteLeft), a

	// spriteRight=spriteLeft+spriteWidth-1;

	ld hl,#_spriteWidth
	add a,(hl)
	dec a
	ld (_spriteRight),a

	// spriteLeftVisible=0;
	// spriteRightVisible=0;

	ld	hl,#_spriteLeftVisible
	ld	(hl), #0x00
	ld	hl,#_spriteRightVisible
	ld	(hl), #0x00

	// if (spriteLeft>=viewLeftOffset)
	// 	if (spriteLeft<=viewRight)
	// 		spriteLeftVisible=1;

	ld	hl, #_viewLeftOffset
	ld a,(_spriteLeft)
	ld b,a
	sub	a, (hl)
	jr c,A0

	ld	a, (#_viewRight)
	sub a,b
	jr c,A0

	ld	hl,#_spriteLeftVisible
	ld	(hl), #0x01

	A0:

	// if (spriteRight>=viewLeftOffset)
	// 	if (spriteRight<=viewRight)
	// 		spriteRightVisible=1;

	ld	hl, #_viewLeftOffset
	ld a,(_spriteRight)
	ld b,a
	sub	a, (hl)
	jr c,A1

	ld	a, (#_viewRight)
	sub a,b
	jr c,A1

	ld	hl,#_spriteRightVisible + 0
	ld	(hl), #0x01

	A1:

	ld	a,(_spriteLeftVisible)
	dec	a
	jr	Z,PS1
	ld	a,(_spriteRightVisible)
	dec	a
	ret	NZ

PS1:
	// spriteHeight=znSprite->height;

	ld	hl, (_znSprite1)
	inc	hl
	inc	hl
	ld	a, (hl)
	ld	(#_spriteHeight),a

	// spriteTop=znInst_tmp->y+yOffset;

	ld	hl, (_znInst1)
	ld	bc, #0x0004
	add	hl, bc
	ld	a, (hl)
	ld	hl, #_yOffset
	add	a, (hl)
	ld	(_spriteTop), a

	// spriteBottom=spriteTop+spriteHeight-1;

	ld hl,#_spriteHeight
	add a,(hl)
	dec a
	ld (_spriteBottom),a

	// spriteTopVisible=0;
	// spriteBottomVisible=0;

	ld	hl,#_spriteTopVisible
	ld	(hl), #0x00
	ld	hl,#_spriteBottomVisible
	ld	(hl), #0x00

	// if (spriteTop>=viewTopOffset)
	// 	if (spriteTop<=viewBottom)
	// 		spriteTopVisible=1;

	ld	hl, #_viewTopOffset
	ld	a, (_spriteTop)
	ld b,a
	sub	a, (hl)
	jr c,B0

	ld	a, (#_viewBottom)
	sub	a,b
	jr c,B0

	ld	hl,#_spriteTopVisible + 0
	ld	(hl), #0x01

	B0:

	// if (spriteBottom>=viewTopOffset)
	// 	if (spriteBottom<=viewBottom)
	// 		spriteBottomVisible=1;

	ld	hl, #_viewTopOffset
	ld	a, (_spriteBottom)
	ld b,a
	sub	a, (hl)
	jr c,B1

	ld	a, (#_viewBottom)
	sub	a,b
	jr c,B1

	ld	hl,#_spriteBottomVisible + 0
	ld	(hl), #0x01

	B1:

	ld	a,(_spriteTopVisible)
	dec	a
	jr	Z,PS2
	ld	a,(_spriteBottomVisible)
	dec	a
	ret	NZ

PS2:
	// pBuffer=pBufferFront;

	ld	hl, (_pBufferFront)
	ld	(_pBuffer), hl

	// pTileList=znSprite->tileList;

	ld	hl, (_znSprite1)
	ld	bc, #0x0009
	add	hl, bc
	ld	a, (hl)
	ld	(_pTileList), a
	inc	hl
	ld	a, (hl)
	ld	(_pTileList+1), a

	ld	a,(_spriteLeftVisible)
	or	a, a
	jr	NZ,PS3

	// pTileList+=(viewLeftOffset-spriteLeft);

	ld a,(_viewLeftOffset)
	ld hl,#_spriteLeft
	sub (hl)
	ld b,#0
	ld c,a

	ld hl,(_pTileList)
	add hl,bc
	ld (_pTileList),hl

	// spriteLeft=viewLeftOffset;

	ld	a,(_viewLeftOffset)
	ld	(_spriteLeft),a

	jr	PS4

PS3:
	// pBuffer+=(spriteLeft-viewLeftOffset);

	ld a,(_spriteLeft)
	ld hl,#_viewLeftOffset
	sub (hl)
	ld b,#0
	ld c,a

	ld hl,(_pBuffer)
	add hl,bc
	ld (_pBuffer),hl

	// if (spriteRightVisible==0)
	// 	spriteRight=viewRight;

	ld	a,(_spriteRightVisible)
	or	a
	jr nz,PS4

	ld	a,(_viewRight)
	ld	(_spriteRight),a

PS4:
	ld	a,(_spriteTopVisible)
	or	a, a
	jr	NZ,PS5

	// pTileList+=(viewTopOffset-spriteTop)*spriteWidth;
	// spriteTop=viewTopOffset;

	ld a,(_viewTopOffset)
	ld hl,#_spriteTop
	sub a,(hl)

	ld hl,#0
	ld de,(_spriteWidth)
	ld d,#0
	ld b,a

	F0:
	add hl,de
	djnz F0

	ld bc,(_pTileList)
	add hl,bc
	ld (_pTileList),hl

	ld	a,(_viewTopOffset)
	ld	(_spriteTop),a

	jp	_printSprite

PS5:
	// pBuffer+=(spriteTop-viewTopOffset)*BUFFER_WIDTH;

	ld a,(_spriteTop)
	ld hl,#_viewTopOffset
	sub a,(hl)

	ld	c,a
	ld	b, #0x00
	ld	l, c
	ld	h, b
	add	hl, hl
	add	hl, hl
	add	hl, bc
	add	hl, hl
	add	hl, hl
	add	hl, hl

	ld bc,(_pBuffer)
	add hl,bc
	ld (_pBuffer),hl

	// if (spriteBottomVisible==0)
	// 	spriteBottom=viewBottom;

	ld	a,(_spriteBottomVisible)
	or	a, a
	jp nz,_printSprite

	ld	a,(_viewBottom)
	ld	(_spriteBottom),a

	jp  _printSprite

	__endasm;
}

void placeObjectsC(){
	// znCount1=zn1->spriteCount;

	__asm
	ld	hl, (_zn1)
	ld	a, (hl)
	ld	(_znCount1), a

	// if (znCount1!=0)

	or	a, a
	jr	Z,C3

	// znInst1=zn1->zoneInstance;

	ld	hl, (_zn1)
	inc	hl
	ld	a, (hl)
	ld	(_znInst1), a
	inc	hl
	ld	a, (hl)
	ld	(_znInst1+1), a

	// u8 i;

	// for (i=0;i<znCount1;i++){
	// 	// if (znInst1->active==1){
	// 	// 	znInst_tmp=znInst1;
	// 	// 	processSprite();
	// 	// }

	// 	a();
	// 	// znInst1++;
	// }

	ld	c, #0x00

C2:
	ld	hl, #_znCount1
	ld	a, c
	sub	a, (hl)
	jr NC,C3

	push	bc

	// if (znInst1->active==1){
	// 	znInst_tmp=znInst1;
	// 	processSprite();
	// }

	ld	hl, (_znInst1)
	ld	c, (hl)
	dec	c
	jr	NZ,A2

	call  _processSprite

	A2:

	// znInst1++;

	ld hl,(_znInst1)
	ld bc,#10
	add hl,bc
	ld (_znInst1),hl

	pop	bc

	inc	c
	jr	C2

	C3:
	__endasm;
}

void placeObjectsA(){
	// u8 x;

	// for (x=0;x<5;x++){
	// 	c();
	// 	pX1+=32;
	// }

	__asm
	// ld	c, #0x05
	ld	c, #0x03

POA:
	push	bc

	// tmp8=pX1/32;
	// tmp8+=pZone1;
	// zn1=zone_permanent+tmp8;

	ld	a,(_pX1)
	rlca
	rlca
	rlca
	and	a, #0x07

	ld	hl, #_pZone1
	add	a, (hl)

	ld	bc, #_zone_permanent
	ld	d, #0x00
	ld e,a
	ld	l, e
	ld	h, d
	add	hl, hl
	add	hl, de
	add	hl, bc
	// ld	(_zn1), hl

	// znCount1=zn1->spriteCount;

	// ld	hl, (_zn1)
	ld	a, (hl)
	ld	(_znCount1), a

	// if (znCount1!=0)

	or	a, a
	jr	Z,C1

	// znInst1=zn1->zoneInstance;

	// ld	hl, (_zn1)
	inc	hl
	ld	a, (hl)
	ld	(_znInst1), a
	inc	hl
	ld	a, (hl)
	ld	(_znInst1+1), a

	// u8 i;

	// for (i=0;i<znCount1;i++){
	// 	b();
	// }

	ld	c, #0x00

C0:
	ld	hl, #_znCount1
	ld	a, c
	sub	a, (hl)
	jr NC,C1

	push	bc

	// znInst_tmp=znInst1;
	// processSprite();

	call  _processSprite

	// znInst1++;

	ld hl,(_znInst1)
	ld bc,#10
	add hl,bc
	ld (_znInst1),hl

	pop	bc

	inc	c
	jr	C0

C1:
	pop	bc

	ld	hl, #_pX1
	ld	a, (hl)
	add	a, #0x20
	ld	(hl), a

	dec c
	jr	NZ,POA
	__endasm;
}

void placeObjectsB(){
	// u8 x;

	// for (x=0;x<5;x++){
	// 	a1();
	// 	b();

	// 	a2();
	// 	b();

	// 	a3();
	// 	b();

	// 	pX1+=32;
	// }

	__asm
	// ld	c, #0x05
	ld	c, #0x03

POB:
	push bc

	// // zn1=zone_respawn+pZone1+pX1/32;

	// ld	a,(_pX1)
	// rlca
	// rlca
	// rlca
	// and	a, #0x07

	// ld	hl, #_pZone1
	// add	a, (hl)

	// ld	bc, #_zone_respawn
	// ld	d, #0x00
	// ld e,a
	// ld	l, e
	// ld	h, d
	// add	hl, hl
	// add	hl, de
	// add	hl, bc
	// ld	(_zn1), hl

	// call _placeObjectsC

	// zn1=zone_group+pZone1+pX1/32;

	ld	a,(_pX1)
	rlca
	rlca
	rlca
	and	a, #0x07

	ld	hl, #_pZone1
	add	a, (hl)

	ld	bc, #_zone_group
	ld	d, #0x00
	ld e,a
	ld	l, e
	ld	h, d
	add	hl, hl
	add	hl, de
	add	hl, bc
	ld	(_zn1), hl

	call _placeObjectsC

	// // zn1=zone_toggle+pZone1+pX1/32;

	// ld	a,(_pX1)
	// rlca
	// rlca
	// rlca
	// and	a, #0x07

	// ld	hl, #_pZone1
	// add	a, (hl)

	// ld	bc, #_zone_toggle
	// ld	d, #0x00
	// ld e,a
	// ld	l, e
	// ld	h, d
	// add	hl, hl
	// add	hl, de
	// add	hl, bc
	// ld	(_zn1), hl

	// call _placeObjectsC

	ld	hl, #_pX1
	ld	a, (hl)
	add	a, #0x20
	ld	(hl), a

	pop bc
	dec c
	jr	NZ,POB
	__endasm;
}

void placeObjects(){
	u8 y;
	u8 pY=pTop-32;
	
	pTmp=pTop/32;

	// if (pTop>=(pTmp*32+16))
	if (pTop>=pTmpOffset())
		pY=pTop;

	viewLeftOffset=viewLeft+xOffset;
	viewRight=viewLeftOffset+BUFFER_WIDTH-1;
	viewTopOffset=viewTop+yOffset;
	viewBottom=viewTopOffset+BUFFER_HEIGHT-1;

	// for (y=0;y<3;y++){
	for (y=0;y<2;y++){
		// pX1=pLeft-64;
		pX1=pLeft-32;
		// pTmp=pLeft/32;

		// if (pLeft>=(pTmp*32+16))
		// 	pX1=pLeft;

		pZone1=(pY/32)*8;
		placeObjectsA();
		// pX1=pLeft-64;

		pX1=pLeft-32;
		// pTmp=pLeft/32;

		// if (pLeft>=(pTmp*32+16))
		// 	pX1=pLeft;

		placeObjectsB();
		pY+=32;
	}
}

void rotateBuffers(){
	if (buffer==0){
		pBufferFront=buffer2;
		pBufferBack=buffer1;
		buffer=2;
	}
	else if (buffer==1){
		pBufferFront=buffer0;
		pBufferBack=buffer2;
		buffer=0;
	}
	else{
		pBufferFront=buffer1;
		pBufferBack=buffer0;
		buffer=1;
	}
}

void flipScreen(){
	// cpct_waitVSYNC();

	if (page==0){
		cpct_setVideoMemoryPage(cpct_pageC0);
		pScreen=SCREEN_BUFFER+64;
		page=1;
	}
	else{
		cpct_setVideoMemoryPage(cpct_page80);
		pScreen=CPCT_VMEM_START+64;
		page=0;
	}
}

void collisionReact(){
	cpct_akp_SFXPlay(4,15,48,0,0,AY_CHANNEL_C);

	switch (cDir){
		case COLLISION_UP:
			pDir=13;
			pTop++;
			return;

		case COLLISION_UP_RIGHT:
			if (pRight==sLeft){
				if (pColTop==sBottom)
					pDir=7;
				else
					pDir=1;
			}
			else
				pDir=7;

			pLeft--;
			pTop++;
			return;

		case COLLISION_RIGHT:
			pDir=19;
			pLeft--;
			return;

		case COLLISION_DOWN_RIGHT:
			if (pRight==sLeft){
				if (pBottom==sTop)
					pDir=7;
				else
					pDir=13;
			}
			else
				pDir=7;

			pLeft--;
			pTop--;
			return;

		case COLLISION_DOWN:
			pDir=1;
			pTop--;
			return;

		case COLLISION_DOWN_LEFT:
			if (pColLeft==sRight){
				if (pBottom==sTop)
					pDir=19;
				else
					pDir=13;
			}
			else
				pDir=19;

			pLeft++;
			pTop--;
			return;

		case COLLISION_LEFT:
			pDir=7;
			pLeft++;
			return;

		case COLLISION_UP_LEFT:
			if (pColLeft==sRight){
				if (pColTop==sBottom)
					pDir=19;
				else
					pDir=1;
			}
			else
				pDir=19;

			pLeft++;
			pTop++;
			return;

		// case COLLISION_JUMP_UP:
		// 	pState=STATE_FALL_DOWN;
		// 	pDir=13;
		// 	pTop++;
		// 	return;

		// case COLLISION_JUMP_UP_LEFT:
		// 	if (pColLeft==sRight){
		// 		pState=STATE_JUMP_UP_RIGHT;
		// 		pDir=3+(23-pDir);
		// 	}
			// else{
			// 	pState=STATE_FALL_DOWN_LEFT;
			// 	pDir=16;
			// 	pTop++;
			// }

			// pLeft++;
			// return;

		// case COLLISION_JUMP_UP_RIGHT:
		// 	if (pRight==sLeft){
		// 		pState=STATE_JUMP_UP_LEFT;
		// 		pDir=23-(pDir-3);
		// 	}
			// else{
			// 	pState=STATE_FALL_DOWN_RIGHT;
			// 	pDir=10;
			// 	pTop++;
			// }

			// pLeft--;
			// return;

		// case COLLISION_FALL_DOWN_LEFT:
		// 	pState=STATE_FALL_DOWN_RIGHT;
		// 	pDir=10;
		// 	pLeft++;
		// 	return;

		// case COLLISION_FALL_DOWN_RIGHT:
		// 	pState=STATE_FALL_DOWN_LEFT;
		// 	pDir=16;
		// 	pLeft--;
		// 	return;
	}
}

void prepareStaticScreen(){
    cpct_setBorder(HW_BLUE);
 	cpct_setPalette(hw_palette_invisible,16);
	cpct_setVideoMemoryPage(cpct_pageC0);
 	cpct_clearScreen_f64(0);
 	cpct_setPalette(hw_palette_visible,16);
}

void menuInit(){
	silence();
	gameState=GAME_STATE_MENU;
 	titlePhase=0;
 	titlePhaseDir=1;
 	prepareStaticScreen();
	printMenu();
}

void instructionsInit(){
	gameState=GAME_STATE_INSTRUCTIONS;
 	prepareStaticScreen();
	printInstructions();
}

void creditsInit(){
	gameState=GAME_STATE_CREDITS;
 	prepareStaticScreen();
	printCredits();
}

void gameOverLoseInit(){
	gameState=GAME_STATE_GAME_OVER_LOSE;
 	prepareStaticScreen();
	printGameOverLose();
}

void gameOverWinInit(){
	u8 i;

	for (i=0;i<4;i++)
		score[i]=0;

	gameState=GAME_STATE_GAME_OVER_WIN;
 	prepareStaticScreen();
	printGameOverWin();
}

void printChunkyWord(u16 vFrontOffset,u8 bytes){
	u8* vmemFront=CPCT_VMEM_START+vFrontOffset;
	u8* vmemBack=SCREEN_BUFFER;
	u8 i;

	for (i=0;i<8;i++){
		cpct_memcpy(vmemFront,vmemBack,bytes);
		vmemFront+=0x0800;
		cpct_memcpy(vmemFront,vmemBack,bytes);
		vmemBack+=0x0800;

		if (i==3)
			vmemFront-=0x3800-64;
		else
			vmemFront+=0x0800;
	}
}

void updateHighScore(){
	u16 hs;
	u16 s;
	u8 i;

	hs=highScore[0]+highScore[1]*10+highScore[2]*100+highScore[3]*1000+highScore[4]*10000;
	s=score[0]+score[1]*10+score[2]*100+score[3]*1000+score[4]*10000;

	if (s>hs){
		for (i=0;i<5;i++)
			highScore[i]=score[i];
	}
}

void gameInit(){
	u8 i;

	gameState=GAME_STATE_GAME;
	pScreen=CPCT_VMEM_START+64;
	page=0;
	buffer=0;
	pBufferFront=buffer0;
	pBufferBack=buffer2;

	pLeft=0;
	pTop=36;
	pDir=7;
	pRot=0;
	pState=STATE_SWIM;
	pPhase=0;
	pJumpTurn=0;

	cpct_memset_f64(SCREEN_BUFFER,0x0000,0x4000);
	cpct_memset_f64(buffer1,0xFFFF,BUFFER_WIDTH*BUFFER_HEIGHT*2); // fill buffer1 and buffer2

	scoreInit();

 	cpct_setPalette(hw_palette_invisible,16);
	cpct_setVideoMemoryPage(cpct_pageC0);
 	cpct_clearScreen_f64(0);

	energy=PLAYER_ENERGY_MAX;
	pEnergyReps=64;
	printEnergy();

	vmem=SCREEN_BUFFER+64*31;

	for (i=0;i<8;i++){
		cpct_memset_f64(vmem,0x3C3C,64);
		cpct_memset_f64(vmem+0x4000,0x3C3C,64);
		vmem+=0x0800;
	}

	printWord_score();

	__asm
	ld hl,#_lz_goatfish_lz
	call printWord_full
	__endasm;

	printWord_high();
	printHighScore();

	cpct_waitVSYNC();
 	cpct_setPalette(hw_palette_visible,16);

	startupActive();
	framesToDisplayFoodScore=0;

	silence();
    cpct_akp_musicInit(sfx_bin);
    cpct_akp_SFXInit(sfx_bin);
	// fxChannel=AY_CHANNEL_C;
	fxNoteIndex=0;
	// fxGroupIndex=0;
	fxCountdown=0;
	fxLastNote=255;
}

// void checkActive_respawn(){
// 	__asm
// 	ld hl,(_znInst4)
// 	inc hl
// 	dec (hl) // znInstance->inactiveCycles--
// 	ld a,(hl)
// 	or a
// 	ret nz
// 	dec hl
// 	ld a,#1
// 	ld (hl),a // znInstance->active=1
// 	// ld (_respawnedSomething),a
// 	__endasm;
// }

// void a2(){
// 	// zn2=zone_respawn+pZone2+pX2/32;
// 	// znCount6=zn2->spriteCount;

// 	__asm

// 	ld	a,(_pX2)
// 	rlca
// 	rlca
// 	rlca
// 	and	a, #0x07

// 	ld	hl, #_pZone2
// 	add	a, (hl)

// 	ld	bc, #_zone_respawn
// 	ld	d, #0x00
// 	ld e,a
// 	ld	l, e
// 	ld	h, d
// 	add	hl, hl
// 	add	hl, de
// 	add	hl, bc
// 	ld	(_zn2), hl

// 	ld	a, (hl)
// 	ld	(_znCount6),a

// 	// u8 i;

// 	// if (znCount6!=0){
// 	// 	znInst4=zn2->zoneInstance;

// 	// 	for (i=0;i<znCount6;i++){
// 	// 		if (znInst4->active==0)
// 	// 			checkActive_respawn();

// 	// 		znInst4++;
// 	// 	}
// 	// }

// 	// pX2+=32;

// 	// ld	a,(#_znCount6 + 0)
// 	or	a, a
// 	jr	Z,CR105

// 	// ld	hl, (_zn2)
// 	inc	hl
// 	ld	a, (hl)
// 	// ld	iy, #_znInst4
// 	// ld	0 (iy), a
// 	ld (_znInst4),a
// 	inc	hl
// 	ld	a, (hl)
// 	// ld	1 (iy), a
// 	ld (_znInst4+1),a

// 	ld	c, #0x00

// CR107:
// 	ld	hl, #_znCount6
// 	ld	a, c
// 	sub	a, (hl)
// 	jr	NC,CR105

// 	ld	hl, (_znInst4)
// 	ld	a, (hl)
// 	or	a, a
// 	jr	NZ,CR102

// 	// push	bc
// 	call	_checkActive_respawn
// 	// pop	bc

// CR102:
// 	// ld	hl, #_znInst4
// 	// ld	a, (hl)
// 	// add	a, #0x0a
// 	// ld	(hl), a
// 	// inc	hl
// 	// ld	a, (hl)
// 	// adc	a, #0x00
// 	// ld	(hl), a

// 	ld hl,(_znInst4)
// 	ld de,#10
// 	add hl,de
// 	ld (_znInst4),hl

// 	inc	c
// 	jr	CR107

// CR105:
// 	ld	hl, #_pX2
// 	ld	a, (hl)
// 	add	a, #0x20
// 	ld	(hl), a

// 	__endasm;
// }

// void check_respawn(){
// 	u8 x;
// 	u8 y;

// 	u8 pY=pTop-64;

// 	for (y=0;y<5;y++){
// 		pX2=pLeft-64;
// 		pZone2=(pY/32)*8;

// 		for (x=0;x<5;x++)
// 			a2();

// 		pY+=32;
// 	}
// }

// void a3(){
// 	// zn2=zone_toggle+pZone2+pX2/32;
// 	// znCount6=zn2->spriteCount;

// 	__asm

// 	ld	a,(_pX2)
// 	rlca
// 	rlca
// 	rlca
// 	and	a, #0x07

// 	ld	hl, #_pZone2
// 	add	a, (hl)

// 	ld	bc, #_zone_toggle
// 	ld	d, #0x00
// 	ld e,a
// 	ld	l, e
// 	ld	h, d
// 	add	hl, hl
// 	add	hl, de
// 	add	hl, bc
// 	ld	(_zn2), hl

// 	ld	a, (hl)
// 	ld	(_znCount6),a

// 	// u8 i;

// 	// if (znCount6!=0){
// 	// 	znInst4=zn2->zoneInstance;

// 	// 	for (i=0;i<znCount6;i++){
// 	// 		if (znInst4->inactiveCycles!=0){
// 	// 			__asm
// 	// 			ld hl,(_znInst4)
// 	// 			inc hl
// 	// 			dec (hl) // znInst4->inactiveCycles--
// 	// 			__endasm;
// 	// 		}
// 	// 		else{
// 	// 			toggleCycles=znInst4->toggleCycles;

// 	// 			if (znInst4->active==1){
// 	// 				__asm
// 	// 				ld hl,(_znInst4)
// 	// 				ld (hl),#0 // znInst4->active=0
// 	// 				ld a,(_toggleCycles)
// 	// 				inc hl
// 	// 				ld (hl),a // znInst4->inactiveCycles=znInst4->toggleCycles
// 	// 				__endasm;
// 	// 			}
// 	// 			else{
// 	// 				__asm
// 	// 				ld hl,(_znInst4)
// 	// 				ld (hl),#1 // znInst4->active=1
// 	// 				ld a,(_toggleCycles)
// 	// 				inc hl
// 	// 				ld (hl),a // znInst4->inactiveCycles=znInst4->toggleCycles
// 	// 				__endasm;
// 	// 			}
// 	// 		}

// 	// 		znInst4++;
// 	// 	}
// 	// }

// 	// pX2+=32;

// 	ld	a,(#_znCount6 + 0)
// 	or	a, a
// 	jr	Z,A5109

// 	ld	hl, (_zn2)
// 	inc	hl
// 	ld	a, (hl)
// 	ld (_znInst4),a
// 	inc	hl
// 	ld	a, (hl)
// 	ld (_znInst4+1),a

// 	ld	c, #0x00

// A5111:
// 	ld	hl, #_znCount6
// 	ld	a, c
// 	sub	a, (hl)
// 	jr	NC,A5109

// 	ld	de, (_znInst4)
// 	ld	l, e
// 	ld	h, d
// 	inc	hl
// 	ld	a, (hl)
// 	or	a, a
// 	jr	Z,A5105

// 	ld	hl,(_znInst4)
// 	inc	hl
// 	dec	(hl)
// 	jr	A5106

// A5105:
// 	ld	l, e
// 	ld	h, d
// 	inc	hl
// 	inc	hl
// 	inc	hl
// 	inc	hl
// 	inc	hl
// 	ld	a, (hl)
// 	ld	(_toggleCycles), a

// 	ld	a, (de)
// 	dec	a
// 	jr	NZ,A5102

// 	ld	hl,(_znInst4)
// 	ld	(hl),#0
// 	ld	a,(_toggleCycles)
// 	inc	hl
// 	ld	(hl),a
// 	jr	A5106

// A5102:
// 	ld	hl,(_znInst4)
// 	ld	(hl),#1
// 	ld	a,(_toggleCycles)
// 	inc	hl
// 	ld	(hl),a

// A5106:
// 	ld hl,(_znInst4)
// 	ld de,#10
// 	add hl,de
// 	ld (_znInst4),hl

// 	inc	c
// 	jr	A5111

// A5109:
// 	ld	hl, #_pX2
// 	ld	a, (hl)
// 	add	a, #0x20
// 	ld	(hl), a

// 	__endasm;
// }

// void check_toggle(){
// 	u8 x;
// 	u8 y;

// 	u8 pY=pTop-32;

// 	for (y=0;y<3;y++){
// 		pX2=pLeft-64;
// 		pZone2=(pY/32)*8;

// 		for (x=0;x<5;x++)
// 			a3();

// 		pY+=32;
// 	}
// }

void a4(){
	// zn2=zone_group+pZone2+pX2/32;
	// znCount6=zn2->spriteCount;

	__asm

	ld	a,(_pX2)
	rlca
	rlca
	rlca
	and	a, #0x07

	ld	hl, #_pZone2
	add	a, (hl)

	ld	bc, #_zone_group
	ld	d, #0x00
	ld e,a
	ld	l, e
	ld	h, d
	add	hl, hl
	add	hl, de
	add	hl, bc
	ld	(_zn2), hl

	ld	a, (hl)
	ld	(_znCount6),a

	__endasm;	
}

void a5(){
	// u8 i;

	// if (znCount6!=0){
	// 	znInst3=zn2->zoneInstance;

	// 	for (i=0;i<znCount6;i++){
	// 		if (znInst3->group==gp){
	// 			__asm
	// 			ld hl,(_znInst3)
	// 			ld (hl),#1 // znInst3->active=1
	// 			__endasm;
	// 		}

	// 		znInst3++;
	// 	}
	// }

	// pX2+=32;

	__asm

	ld	a,(#_znCount6 + 0)
	or	a, a
	jr	Z,A5105a

	ld	hl, (_zn2)
	inc	hl
	ld	a, (hl)
	ld (_znInst3),a
	inc	hl
	ld	a, (hl)
	ld (_znInst3+1),a

	ld	c, #0x00

A5107a:
	ld	hl, #_znCount6
	ld	a, c
	sub	a, (hl)
	jr	NC,A5105a

	ld	hl, (_znInst3)
	ld	de, #0x0008
	add	hl, de
	ld	b, (hl)
	inc	hl
	ld	e, (hl)
	ld a,(_gp)
	sub	a, b
	jr	NZ,A5102a
	ld a,(_gp+1)
	sub	a, e
	jr	NZ,A5102a

	ld	hl,(_znInst3)
	ld	(hl),#1

A5102a:
	ld hl,(_znInst3)
	ld de,#10
	add hl,de
	ld (_znInst3),hl

	inc	c
	jr	A5107a

A5105a:
	ld	hl, #_pX2
	ld	a, (hl)
	add	a, #0x20
	ld	(hl), a

	__endasm;
}

void activateGroup(){
	u8 x;
	u8 y;
	u8 pY=pTop-32;

	// pTmp=pTop/32;

	// if (pTop>=(pTmp*32+16))
	// if (pTop>=pTmpOffset())
	// 	pY=pTop;

	gp_tmp=gp;

	__asm
	ld hl,(_gp_tmp)
	inc hl
	inc hl
	ld a,(hl) // a=gp->instanceCount
	dec hl
	dec hl
	ld (hl),a // gp->activeCount=gp->instanceCount
	__endasm;

	for (y=0;y<3;y++){
	// for (y=0;y<2;y++){
		// pX2=pLeft-32;
		// pX2=pLeft-64;
		pX2=pLeft-32;
		pZone2=(pY/32)*8;
		// pTmp=pLeft/32;

		// if (pLeft>=pTmpOffset())
		// 	pX2=pLeft;

		// for (x=0;x<3;x++){
		// for (x=0;x<5;x++){
		for (x=0;x<3;x++){
		// for (x=0;x<2;x++){
			a4();
			a5();
		}

		pY+=32;
	}
}

void a6(){
	// u8 i;

	// if (znCount6!=0){
	// 	znInst3=zn2->zoneInstance;

	// 	for (i=0;i<znCount6;i++){
	// 		if (znInst3->group==gp){
	// 			__asm
	// 			ld hl,(_znInst3)
	// 			ld (hl),#0 // znInst3->active=0
	// 			__endasm;
	// 		}

	// 		znInst3++;
	// 	}
	// }

	// pX2+=32;

	__asm

	ld	a,(#_znCount6 + 0)
	or	a, a
	jr	Z,A6105

	ld	hl, (_zn2)
	inc	hl
	ld	a, (hl)
	ld (_znInst3),a
	inc	hl
	ld	a, (hl)
	ld (_znInst3+1),a

	ld	c, #0x00

A6107:
	ld	hl, #_znCount6
	ld	a, c
	sub	a, (hl)
	jr	NC,A6105

	ld	hl, (_znInst3)
	ld	de, #0x0008
	add	hl, de
	ld	b, (hl)
	inc	hl
	ld	e, (hl)
	ld a,(_gp)
	sub	a, b
	jr	NZ,A6102
	ld a,(_gp+1)
	sub	a, e
	jr	NZ,A6102

	ld	hl,(_znInst3)
	ld	(hl),#0

A6102:
	ld hl,(_znInst3)
	ld de,#10
	add hl,de
	ld (_znInst3),hl

	inc	c
	jr	A6107

A6105:
	ld	hl, #_pX2
	ld	a, (hl)
	add	a, #0x20
	ld	(hl), a

	__endasm;
}

void deactivateGroup(){
	u8 x;
	u8 y;
	u8 pY=pTop-32;
	
	// pTmp=pTop/32;

	// if (pTop>=(pTmp*32+16))
	// if (pTop>=pTmpOffset())
	// 	pY=pTop;

	for (y=0;y<3;y++){
	// for (y=0;y<2;y++){
		// pX2=pLeft-32;
		// pX2=pLeft-64;
		pX2=pLeft-32;
		pZone2=(pY/32)*8;
		// pTmp=pLeft/32;

		// if (pLeft>=(pTmp*32+16))
		// if (pLeft>=pTmpOffset())
		// 	pX2=pLeft;

		// for (x=0;x<3;x++){
		// for (x=0;x<5;x++){
		for (x=0;x<3;x++){
		// for (x=0;x<2;x++){
			a4();
			a6();
		}

		pY+=32;
	}
}

void group_startupActive(){
	gp_tmp=gp;

	__asm
	ld hl,(_gp_tmp)
	inc hl
	ld a,(hl) // A=gp->startupActive
	or a
	ret z
	inc hl
	ld a,(hl) // a=gp->instanceCount
	dec hl
	dec hl
	ld (hl),a // gp->activeCount=gp->instanceCount
	__endasm;
}

void startupActive(){
	u8 z;
	u8 i;
	u8 znSpriteCount;
	const typeZone* zn;
	const typeZoneInstance* znInstance;

	// zn=zone_respawn;

	// for (z=0;z<64;z++){
	// 	znSpriteCount=zn->spriteCount;

	// 	if (znSpriteCount!=0){
	// 		znInstance=zn->zoneInstance;

	// 		for (i=0;i<znSpriteCount;i++){
	// 			znInst2=znInstance;

	// 			__asm
	// 			ld hl,(_znInst2)
	// 			inc hl
	// 			inc hl
	// 			ld a,(hl) // A=znInstance->startupActive

	// 			dec hl
	// 			ld (hl),#0 // znInstance->inactiveCycles=0

	// 			dec hl
	// 			ld (hl),a // znInstance->active=znInstance->startupActive
	// 			__endasm;

	// 			znInstance++;
	// 		}
	// 	}

	// 	zn++;
	// }

	for (i=0;i<GROUP_COUNT;i++){
		gp=tblGroup[i];
		group_startupActive();
	}

	zn=zone_group;

	for (z=0;z<64;z++){
		znSpriteCount=zn->spriteCount;

		if (znSpriteCount!=0){
			znInstance=zn->zoneInstance;

			for (i=0;i<znSpriteCount;i++){
				znInst2=znInstance;

				if (znInstance->group->startupActive==1){
					__asm
					ld hl,(_znInst2)
					ld (hl),#1 // znInstance->active=1
					__endasm;
				}
				else{
					__asm
					ld hl,(_znInst2)
					ld (hl),#0 // znInstance->active=0
					__endasm;
				}

				znInstance++;
			}
		}

		zn++;
	}

	// zn=zone_toggle;

	// for (z=0;z<64;z++){
	// 	znSpriteCount=zn->spriteCount;

	// 	if (znSpriteCount!=0){
	// 		znInstance=zn->zoneInstance;

	// 		for (i=0;i<znSpriteCount;i++){
	// 			toggleCycles=znInstance->toggleCycles;
	// 			znInst2=znInstance;

	// 			__asm
	// 			ld hl,(_znInst2)
	// 			ld (hl),#1 // znInstance->active=1
	// 			ld a,(_toggleCycles)
	// 			inc hl
	// 			ld (hl),a // znInstance->inactiveCycles=znInstance->toggleCycles
	// 			__endasm;

	// 			znInstance++;
	// 		}
	// 	}

	// 	zn++;
	// }
}

void printRedYellowGreen(){
	__asm
	ld b,#0
	ld a,(_pEnergyRed)
	cp b
	ret z

	ld hl,(_vmem)
	ld (hl),#0xCC

	dec a
	ret z

	ld d,h
	ld e,l
	inc e
	ld c,a
	ldir

	ld b,#0
	ld a,(_pEnergyYellow)
	cp b
	ret z

	ld hl,(_vmem)
	ld c,#16
	add hl,bc
	ld (hl),#0xC3

	dec a
	ret z

	ld d,h
	ld e,l
	inc e
	ld c,a
	ldir

	ld b,#0
	ld a,(_pEnergyGreen)
	cp b
	ret z

	ld hl,(_vmem)
	ld c,#32
	add hl,bc
	ld (hl),#0xF0

	dec a
	ret z

	ld d,h
	ld e,l
	inc e
	ld c,a
	ldir
	__endasm;
}

void printEnergyBar8th(){
	__asm
	ld hl,(_vmem)
	ld d,h
	ld e,l
	add hl,bc
	ex de,hl
	// ld b,#0
	// ld c,a
	ld bc,#64
	ldir
	__endasm;
}

void printEnergyBar(){
	__asm
	// ld a,(_pEnergyReps)
	ld bc,#0x0800
	call _printEnergyBar8th
	ld bc,#0x1000
	call _printEnergyBar8th
	ld bc,#0x1800
	call _printEnergyBar8th
	ld bc,#0x2000
	call _printEnergyBar8th
	ld bc,#0x2800
	call _printEnergyBar8th
	ld bc,#0x3000
	call _printEnergyBar8th
	ld bc,#0x3800
	call _printEnergyBar8th
	__endasm;
}

void printEnergy(){
	u8 i;
	u8 pEnergy;

	pEnergyRed=0;
	pEnergyYellow=0;
	pEnergyGreen=0;

	pEnergy=(energy+7)/PLAYER_ENERGY_DISPLAY_MULTIPLIER;

	if (pEnergy!=0){
		pEnergyRed=pEnergy;

		if (pEnergy>16){
			pEnergyRed=16;
			pEnergyYellow=pEnergy-16;

			if (pEnergyYellow>16)
				pEnergyYellow=16;

			if (pEnergy>32)
				pEnergyGreen=pEnergy-32;
		}
	}

	if (pEnergyReps==64){
		vmem=SCREEN_BUFFER;
		
		for (i=0;i<2;i++){
			printRedYellowGreen();
			printEnergyBar();
			vmem=CPCT_VMEM_START;
		}

		pEnergyReps=32;
	}
	else{
		if (page==0)
			vmem=CPCT_VMEM_START;
		else
			vmem=SCREEN_BUFFER;

		cpct_memset_f64(vmem,0x3C3C,64); // blue
		printRedYellowGreen();

	// 	if (pEnergy>=40)
	// 		vmem+=32;
	// 	else if (pEnergy>=24)
	// 		vmem+=16;

		printEnergyBar();
	}
}

void printScore(){
	u8 i;

	if (page==0)
		vmem=CPCT_VMEM_START;
	else
		vmem=SCREEN_BUFFER;

	for (i=0;i<5;i++){
		cpct_drawTileAligned2x8(tileNum_0+score[i]*16,vmem+64*31+5+8-i*2);
	}
}

void printFoodScore(){
	if (framesToDisplayFoodScore!=0){
		framesToDisplayFoodScore--;

		if (framesToDisplayFoodScore==0)
			__asm
			ld hl,#_lz_goatfish_lz
			call printWord_full
			__endasm;
	}
}

void printHighScore(){
	u8 i;

	for (i=0;i<5;i++){
		cpct_drawTileAligned2x8(tileNum_0+score[i]*16,CPCT_VMEM_START+64*31+5+8-i*2);
		cpct_drawTileAligned2x8(tileNum_0+score[i]*16,SCREEN_BUFFER+64*31+5+8-i*2);

		cpct_drawTileAligned2x8(tileNum_0+highScore[i]*16,CPCT_VMEM_START+64*31+54+8-i*2);
		cpct_drawTileAligned2x8(tileNum_0+highScore[i]*16,SCREEN_BUFFER+64*31+54+8-i*2);
	}
}

void waitForKey(){
	silence();

	while(cpct_isAnyKeyPressed())
		cpct_scanKeyboard_f();

	cpct_scanKeyboard_f();
}

void printPressAnyKey(){
	printString(5,0,"PRESS ANY KEY",64*28+6,52);
}

void printGameOver(){
	printString(3,0,"GAME OVER",64*2+14,36);
}

void printGoatfish(){
	printString(1,0,"GOATFISH",64*6+16,32);
}

void printMenu(){
	u8 i;

	printString(7,0,"________",64*0+16,32);
	printString(7,0,"________",64*4+16,32);

	printString(1,0,"PART FISH",64*8+14,36);
	printString(1,0,"PART GOAT",64*10+14,36);
	printString(1,0,"ALL GLUTTON",64*12+10,44);

	printString(3,0,"HIGH SCORE",64*16+12,40);
	updateHighScore();

	for (i=0;i<5;i++){
		printChar(9,i*4,highScore[4-i]+48);
	}
	
	printChunkyWord(64*18+22,20);
	printString(1,0,"(I)",64*22+4,12);
	printString(5,0,"nstructions",64*22+16,44);
	printString(1,0,"(C)",64*24+14,12);
	printString(5,0,"redits",64*24+26,24);
	printPressAnyKey();
}

void printInstructions(){
	printString(3,0,"EAT TO SURVIVE!",64*2+2,60);
	printString(1,0,"Joystick/Arrows",64*6+2,60);
	printString(1,0,"LEFT/RIGHT",64*8+12,40);
	printString(3,0,"to TURN",64*10+18,28);
	printString(1,0,"FIRE/SPACE",64*14+12,40);
	printString(3,0,"to SWIM",64*16+18,28);
	printString(1,0,"(P)",64*20+8,12);
	printString(3,0,"to PAUSE",64*20+24,32);
	printString(1,0,"(ESC)",64*22+6,20);
	printString(3,0,"to QUIT",64*22+30,28);
	printPressAnyKey();
}

void printCredits(){
	printString(3,0,"Ervin Pajor",64*2+10,44);
	printString(3,0,"Wayne Psaila",64*4+8,48);
	printString(3,0,"2020",64*6+24,16);
	printString(8,0,"GOATFISH",64*10+16,32);
	printString(8,0,"species created",64*12+2,60);
	printString(8,0,"by Antonia",64*14+12,40);
	printString(9,0,"Urchins by Zoe",64*18+4,56);
	printString(7,0,"Dedicated to",64*22+8,48);
	printChar(3,0,228);
	printChar(3,32,228);
	printString(7,4," D+A+Z ",64*24+14,36);
	printPressAnyKey();
}

void printGameOverScore(){
	u8 i;

	printString(3,0,"SCORE",64*18+22,20);

	for (i=0;i<5;i++){
		printChar(9,i*4,score[4-i]+48);
	}

	printChunkyWord(64*20+22,20);
}

void printGameOverLose(){
	printGameOver();
	printGoatfish();
	printString(1,0,"has run out",64*8+10,44);
	printString(1,0,"of ENERGY",64*10+14,36);
	printChar(3,0,225);
	printChunkyWord(64*14+30,4);
	printGameOverScore();
	printPressAnyKey();
}

void printGameOverWin(){
	printGameOver();
	printGoatfish();
	printString(1,0,"has eaten",64*8+14,36);
	printString(1,0,"ENOUGH",64*10+20,24);
	printChar(7,0,224);
	printChunkyWord(64*14+30,4);
	printGameOverScore();
	printPressAnyKey();
}

void printString(u8 colour,u8 vBackOffset,u8* str,u16 vFrontOffset,u8 bytes){
	cpct_setDrawCharM0(colour,0);
	cpct_drawStringM0(str,SCREEN_BUFFER+vBackOffset);
	printChunkyWord(vFrontOffset,bytes);
}

void printChar(u8 colour,u8 vBackOffset,u8 ascii){
	cpct_setDrawCharM0(colour,0);
	cpct_drawCharM0(SCREEN_BUFFER+vBackOffset,ascii);
}

void printMenuTitle(){
	u8 i;

	cpct_setDrawCharM0(7,0);
	cpct_drawStringM0("GOATFISH",SCREEN_BUFFER);
	printChar(1,titlePhase*4,goatfishAscii[titlePhase]);
	cpct_waitVSYNC();

	// print more than once in order to slow down the flashing white character

	for (i=0;i<12;i++)
		printChunkyWord(64*3+16,32);

	titlePhase+=titlePhaseDir;

	if (titlePhase==0)
		titlePhaseDir=1;
	else if (titlePhase==7)
		titlePhaseDir=-1;
}
